package consume

import (
	"applet/app/cfg"
	"applet/app/db"
	utils2 "applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"time"
)

func EggEnergyTeamAssistanceConsume(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>EggEnergyTeamAssistanceConsume>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(50)
	delivery := ch.Consume(queue.Name, false)

	egg_system_rules.Init(cfg.RedisAddr)
	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			err = handleEggEnergyTeamAssistanceDataConsume(res.Body)
			if err != nil {
				fmt.Println("EggEnergyTeamAssistanceConsume_ERR:::::", err.Error())
				utils2.FilePutContents("EggEnergyTeamAssistance_ERR", utils2.SerializeStr(map[string]interface{}{
					"body": res.Body,
					"err":  err.Error(),
				}))
			}
			//_ = res.Reject(false)
			err = res.Ack(true)
			fmt.Println("err ::: ", err)
		} else {
			panic(errors.New("error getting message"))
		}
	}
}

func handleEggEnergyTeamAssistanceDataConsume(msgData []byte) error {
	time.Sleep(time.Duration(10) * time.Millisecond) //休眠100毫秒
	// 1.解析mq中queue的数据结构体
	var msg *md2.EggEnergyTeamAssistanceReq
	err := json.Unmarshal(msgData, &msg)
	if err != nil {
		return err
	}

	// 2.查询数据库数据用户是否已经助力过
	detailDb := implement.NewEggEnergyAssistanceDetailDb(db.Db)
	exist, err := detailDb.AssistanceDetailExist(msg.Uid, msg.AssistanceUid, msg.SignId)
	if err != nil {
		return err
	}
	now := time.Now()
	if !exist {
		//插入助力数据
		detail := model.EggEnergyAssistanceDetail{
			Uid:         msg.Uid,
			AssistedUid: msg.AssistanceUid,
			SignId:      msg.SignId,
			CreatedAt:   now.Format("2006-01-02 15:04:05"),
			UpdatedAt:   now.Format("2006-01-02 15:04:05"),
		}
		_, err = detailDb.AssistanceDetailInsert(&detail)
		if err != nil {
			return err
		}
	}

	// 3. 变更数据
	assistanceDb := implement.NewEggEnergyTeamAssistanceDb(db.Db)
	date := now.Format("2006-01-02")
	session := db.Db.Where("")
	teamAssistance, err := assistanceDb.EggEnergyTeamAssistanceGetOneByParamsBySession(session, map[string]interface{}{
		"key":   "sign_id",
		"value": msg.SignId,
	})
	if err != nil {
		return err
	}
	if teamAssistance == nil {
		m := model.EggEnergyTeamAssistance{
			Uid:           msg.Uid,
			AssistedNum:   1,
			AssistedTimes: 1,
			RewardScore:   utils2.Float64ToStr(msg.AssistanceValue),
			Date:          date,
			SignId:        msg.SignId,
			CreateAt:      now.Format("2006-01-02 15:04:05"),
			UpdateAt:      now.Format("2006-01-02 15:04:05"),
		}
		_, err = assistanceDb.EggEnergyTeamAssistanceInsert(&m)
		if err != nil {
			return err
		}
	} else {
		teamAssistance.RewardScore = utils2.Float64ToStr(msg.AssistanceValue + utils2.StrToFloat64(teamAssistance.RewardScore))
		//	如果未助力过, 助力人数 + 1 添加助力信息
		if !exist {
			teamAssistance.AssistedNum++
		}
		teamAssistance.AssistedTimes++
		_, err = assistanceDb.EggEnergyTeamAssistanceUpdate(teamAssistance.Id, teamAssistance, "assisted_num", "reward_score", "assisted_times")
		if err != nil {
			return err
		}
	}

	return nil
}
